from ast import literal_eval

from ncellapp.aescipher import AESCipher

class NcellResponse(object):
    #: NcellResponse contains decrypted server response from Ncell and other requests.Response() properties.
    def __init__(self, response, customOp=None, customError=None):
        self.aes = AESCipher()
        self.response = response
        self.customOp = customOp
        self.customError = customError
        
        try:
            self.responseDict = literal_eval(self.aes.decrypt(self.response.text))['businessOutput']
            self.responseDict2 = self.responseDict.copy()
        except AttributeError:
            self.responseDict = None
            self.responseDict2 = None
                       
    def __repr__(self):
        return f'<OperationStatus [{self.opStatus}]>'
    
    @property
    def cacheDataInMins(self):
        try:
            return self.responseDict['cacheDataInMins']
        except AttributeError:
            pass
    
    @property
    def currentDate(self):
        try:
            return self.responseDict['currentDate']
        except AttributeError:
            pass
    
    @property
    def opStatus(self):
        try:
            if self.customOp:
                return self.customOp
            else:
                return self.responseDict['opStatus']
        except AttributeError:
            return None

    @property
    def errorMessage(self):
        try:
            if self.customError:
                return self.customError
            else:
                return self.responseDict['errorMessage']
        except AttributeError:
            pass
        
    @property
    def content(self):
        try:
            toRemove = ['cacheDataInMins','currentDate','opStatus','errorMessage']
            for i in toRemove:
                self.responseDict2.pop(i, None)
            return self.responseDict2
        except AttributeError:
            pass
    
    @property
    def cookies(self):
        try:
            return self.response.cookies
        except AttributeError:
            pass
    
    @property
    def elapsed(self):
        try:
            return self.response.elapsed
        except AttributeError:
            pass
    
    @property
    def headers(self):
        try:
            return self.response.headers
        except AttributeError:
            pass
    
    @property
    def ok(self):
        try:
            return self.response.ok
        except AttributeError:
            pass
    
    @property
    def reason(self):
        try:
            return self.response.reason
        except AttributeError:
            pass
    
    @property
    def request(self):
        try:
            return self.response.request
        except AttributeError:
            pass
    
    @property
    def statusCode(self):
        try:
            return self.response.status_code
        except AttributeError:
            pass
    
    @property
    def url(self):
        try:
            return self.response.url
        except AttributeError:
            pass


import requests
from base64 import (b64encode, b64decode)
from ast import literal_eval
from datetime import datetime

from ncellapp.aescipher import AESCipher
from ncellapp.models import NcellResponse

class NcellApp():
    def __init__(self):
        self.baseUrl = 'http://ssa.ncell.com.np:8080/mc/selfcare/v2/proxy' 
        self.aes = AESCipher()
        self.headers = {
            'X-MobileCare-AppClientVersion': 'SHn7MOIW3T/R/OL8LsAvxw==',
            'Cache-Control': 'no-cache',
            'X-MobileCare-PreferredLocale': 'cAsAM2g0t7oB6OSJKH1ptQ==',
            'Content-Type': 'application/xml',
            'X-MobileCare-APIKey': 'ABC_KEY',
            'X-MobileCare-AppResolution': 'iRRhXh87ipDTZpyEWGWteg==',
            'X-MobileCare-AppPlatformVersion': 'QJ2ZR3DKpuBfBr7GuTQh7w==',
            'ACCEPT': 'application/json',
            'X-MobileCare-AppPlatformName': 'yEHXRN3mrQMvwG4bfE2ApQ==',
            'Host': 'ssa.ncell.com.np:8080',
            'Connection': 'Keep-Alive',
        }
        
class register(NcellApp): 
    #: Register class contains the methods for registering the account.
    #: Every methods returns ncellapp.models.NcellResponse object.
    def __init__(self, msisdn):
        NcellApp.__init__(self)
        self.msisdn = str(msisdn)
        self.deviceClientId = None
    
    def sendOtp(self):
        #: Request Ncell to send OTP to the number for registration
        url = self.baseUrl + '/register'
        
        data = f"<?xml version='1.0' encoding='UTF-8' standalone='yes' ?><mAppData><userOperationData><serviceInstance>{self.msisdn}</serviceInstance></userOperationData></mAppData>"
        data = self.aes.encrypt(data)
        
        response = requests.post(url, headers=self.headers, data=data)
        
        r = NcellResponse(response)
        
        try:
            self.deviceClientId = self.aes.encrypt(r.content['deviceClientId'])
        except KeyError:
            pass
        
        return r
    
    def getToken(self, otp):
        #: Send the OTP to the Ncell server for verification and get token as response if correct
        headers2 = self.headers
        headers2.update({
            'X-MobileCare-DeviceClientID':  self.deviceClientId,
            'X-MobileCare-MSISDN': self.msisdn,          
        })
        
        url = self.baseUrl + '/register'
        
        data = f"<?xml version='1.0' encoding='UTF-8' standalone='yes' ?><mAppData><userOperationData><otp>{otp}</otp></userOperationData></mAppData>"
        data = self.aes.encrypt(data)
        
        response = requests.post(url, headers=headers2, data=data)
        
        r = NcellResponse(response)
        
        if r.opStatus == '0':
            token = b64encode(str({'msisdn':self.msisdn, 'deviceClientId':self.deviceClientId}).encode()).decode()
            r.content.update({'token':token})
          
        return r
         
class ncell(NcellApp):
    #: Ncell class contains the methods for using the features of ncell app.
    #: Every methods returns ncellapp.models.NcellResponse object.  
    def __init__(self, token):
        NcellApp.__init__(self)
        self.token = token
        self.name = self.msisdn = self.status = self.partyID = self.accountId = self.serviceFlag = self.currentPlan = self.secureToken = self.hubID = None
        
    def login(self):
        #: Extract the msisdn and client ID from the token and view the profile to check the token validity and to extract other important information
        try:
            self.msisdn = literal_eval(b64decode(self.token).decode())['msisdn']
            self.deviceClientId = literal_eval(b64decode(self.token).decode())['deviceClientId']
        except Exception:
            r = NcellResponse(None, customOp='invalid', customError='The token you provided is invalid.')
            return r
        
        self.headers.update({
            'X-MobileCare-DeviceClientID': self.deviceClientId,
            'X-MobileCare-MSISDN': self.msisdn,          
        })
        
        profile = self.viewProfile().content
        
        try:
            self.name = profile['myProfile']['name']
            self.msisdn = profile['myProfile']['MSISDN']
            self.status = profile['myProfile']['status']
            self.partyID = profile['myProfile']['partyID']
            self.accountId = profile['myProfile']['accountID']
            self.serviceFlag = profile['myProfile']['serviceFlag']
            self.currentPlan = profile['myProfile']['currentPlan']
            self.secureToken = profile['myProfile']['secureToken']
            self.hubID = profile['myProfile']['hubID']
            
            r = NcellResponse(None, customOp='0', customError='SUCCESS')
            return r
            
        except KeyError:      
            r = NcellResponse(None, customOp='expired', customError='The token you provided has expired.')
            return r
            
    def viewProfile(self):
        #: View the profile of the account
        url = self.baseUrl + '/viewMyProfile'

        data = "<?xml version='1.0' encoding='UTF-8' standalone='yes' ?><mAppData><userOperationData /></mAppData>"
        data = self.aes.encrypt(data)
        
        response = requests.post(url, headers=self.headers, data=data)
        
        r = NcellResponse(response)
        
        return r
    
    def sendSms(self, destination, message, schedule=None):
        #: Send SMS with the currentPlan
        url = self.baseUrl + '/updateServiceRequest'
        schedule = schedule or datetime.now().strftime("%Y%m%d%H%M%S")

        data = f"<?xml version='1.0' encoding='UTF-8' standalone='yes' ?><mAppData><userOperationData><lob>{self.serviceFlag}</lob><userId>{schedule}</userId><problemDesc>{message}</problemDesc><serviceId>SENDSMS</serviceId><accountId>{self.accountId}</accountId><code>{destination}</code><offerId>yes</offerId></userOperationData></mAppData>"
        data = self.aes.encrypt(data)
        
        response = requests.post(url, headers=self.headers, data=data)
        
        r = NcellResponse(response)
        
        return r
    
    def sendFreeSms(self, destination, message, schedule=None):
        #: Send SMS with free 10 SMS plan
        url = self.baseUrl + '/updateServiceRequest'
        schedule = schedule or datetime.now().strftime("%Y%m%d%H%M%S")

        data = f"<?xml version='1.0' encoding='UTF-8' standalone='yes' ?><mAppData><userOperationData><lob>{self.serviceFlag}</lob><userId>{schedule}</userId><problemDesc>{message}</problemDesc><serviceId>SENDSMS</serviceId><accountId>{self.accountId}</accountId><code>{destination}</code><offerId>no</offerId></userOperationData></mAppData>"
        data = self.aes.encrypt(data)
        
        response = requests.post(url, headers=self.headers, data=data)
        
        r = NcellResponse(response)
        
        return r
        
    def viewBalance(self):
        #: View the current balance of the account
        url = self.baseUrl + '/myBalance'
        
        data = f"<?xml version='1.0' encoding='UTF-8' standalone='yes' ?><mAppData><userOperationData><lob>{self.serviceFlag}</lob><contractId></contractId><customerId></customerId><code>{self.accountId}</code><accountId>{self.accountId}</accountId><offerId>{self.hubID}</offerId></userOperationData></mAppData>"
        data = self.aes.encrypt(data)
        
        response = requests.post(url, headers=self.headers, data=data)
        
        r = NcellResponse(response)
        
        return r
    
    def selfRecharge(self, rpin):
        #: Recharging the current account
        url = self.baseUrl + '/updateServiceRequest'
        
        data = f"<?xml version='1.0' encoding='UTF-8' standalone='yes' ?><mAppData><userOperationData><lob>{self.serviceFlag}</lob><alternateContactNumber></alternateContactNumber><contractId></contractId><customerId></customerId><serviceId>RECHARGENOW</serviceId><code>{rpin}</code></userOperationData></mAppData>"
        data = self.aes.encrypt(data)
        
        response = requests.post(url, headers=self.headers, data=data)
        
        r = NcellResponse(response)
        
        return r
    
    def recharge(self, destination, rpin):
        #: Recharging other's account
        url = self.baseUrl + '/updateServiceRequest'
        
        data = f"<?xml version='1.0' encoding='UTF-8' standalone='yes' ?><mAppData><userOperationData><lob>{self.serviceFlag}</lob><alternateContactNumber>{destination}</alternateContactNumber><contractId></contractId><customerId></customerId><serviceId>RECHARGENOW</serviceId><code>{rpin}</code></userOperationData></mAppData>"
        data = self.aes.encrypt(data)
        
        response = requests.post(url, headers=self.headers, data=data)
        
        r = NcellResponse(response)
        
        return r
    
    def rechargeHistory(self):
        #: View the recharge history of the account
        url = self.baseUrl + '/rechargeHistory'
        
        data = f"<?xml version='1.0' encoding='UTF-8' standalone='yes' ?><mAppData><userOperationData><lob>{self.serviceFlag}</lob><contractId></contractId><customerId></customerId><userId>TransferHistory</userId><accountId>{self.accountId}</accountId></userOperationData></mAppData>"
        data = self.aes.encrypt(data)
        
        response = requests.post(url, headers=self.headers, data=data)
        
        r = NcellResponse(response)
        
        return r
    
    def balanceTransfer(self, destination, amount):
        #: Initiate the balance transformation to the destination number
        url = self.baseUrl + '/updateServiceRequest'
        
        data = f"<?xml version='1.0' encoding='UTF-8' standalone='yes' ?><mAppData><userOperationData><lob>{self.serviceFlag}</lob><alternateContactNumber>{destination}</alternateContactNumber><contractId></contractId><customerId></customerId><action>NEW</action><serviceId>BALANCETRANSFER</serviceId><code>{amount}</code></userOperationData></mAppData>"
        data = self.aes.encrypt(data)
        
        response = requests.post(url, headers=self.headers, data=data)
        
        r = NcellResponse(response)
        
        return r
    
    def confirmBalanceTransfer(self, otp):
        #: Confirm the balance transfer
        url = self.baseUrl + '/updateServiceRequest'
        
        data = f"<?xml version='1.0' encoding='UTF-8' standalone='yes' ?><mAppData><userOperationData><lob>{self.serviceFlag}</lob><password>{otp}</password><contractId></contractId><customerId></customerId><action>NEW</action><serviceId>BALANCETRANSFER</serviceId><offerId>validate</offerId></userOperationData></mAppData>"
        data = self.aes.encrypt(data)
        
        response = requests.post(url, headers=self.headers, data=data)
        
        r = NcellResponse(response)
        
        return r
    
    def viewTransaction(self, transactionsFrom, transactionsTo):
        #: Initiate to view call history
        url = self.baseUrl + '/viewTransactions'
        
        self.transactionsFrom = transactionsFrom
        self.transactionsTo = transactionsTo
        
        data = f"<?xml version='1.0' encoding='UTF-8' standalone='yes' ?><mAppData><userOperationData><lob>prepaid</lob><userId>{self.transactionsFrom}</userId><code>GET</code><accountId>{self.accountId}</accountId><offerId>{self.transactionsTo}</offerId></userOperationData></mAppData>"
        data = self.aes.encrypt(data)
        
        response = requests.post(url, headers=self.headers, data=data)
        
        r = NcellResponse(response)
        
        return r
    
    def confirmViewTransaction(self, otp):
        #: Confirm to view call history
        url = self.baseUrl + '/viewTransactions'
        
        data = f"<?xml version='1.0' encoding='UTF-8' standalone='yes' ?><mAppData><userOperationData><lob>prepaid</lob><action>{otp}</action><userId>{self.transactionsFrom}</userId><code>VALIDATE</code><accountId>{self.accountId}</accountId><offerId>{self.transactionsTo}</offerId></userOperationData></mAppData>"
        data = self.aes.encrypt(data)
        
        response = requests.post(url, headers=self.headers, data=data)
        
        r = NcellResponse(response)
        
        return r
    
    def viewService(self, serviceCategory=''):
        #: View the list of available services to activate. Default service category is All.
        url = self.baseUrl + '/viewMyService'
        
        data = f"<?xml version='1.0' encoding='UTF-8' standalone='yes' ?><mAppData><userOperationData><lob>{self.serviceFlag}</lob><contractId></contractId><customerId></customerId><code>R3027</code><serviceCategory>{serviceCategory}</serviceCategory><accountId>{self.accountId}</accountId><offerId>{self.hubID}</offerId></userOperationData></mAppData>"
        data = self.aes.encrypt(data)
        
        response = requests.post(url, headers=self.headers, data=data)
        
        r = NcellResponse(response)
        
        return r
    
    def activateService(self, serviceId):
        #: Activate the certain service. Note: ServiceID is in isMandatory key instead of serviceId of viewService.
        url = self.baseUrl + '/updateServiceRequest'
        
        data = f"<?xml version='1.0' encoding='UTF-8' standalone='yes' ?><mAppData><userOperationData><lob>{self.serviceFlag}</lob><contractId></contractId><customerId></customerId><serviceId>SUBSCRIBEAPRODUCT</serviceId><code>{serviceId}</code></userOperationData></mAppData>"
        data = self.aes.encrypt(data)
        
        response = requests.post(url, headers=self.headers, data=data)
        
        r = NcellResponse(response)
        
        return r
    
    def viewOffer(self):
        #: View the available offer for the account
        url = self.baseUrl + '/viewOffers'
        
        data = f"<?xml version='1.0' encoding='UTF-8' standalone='yes' ?><mAppData><userOperationData><customerId></customerId><lob>{self.serviceFlag}</lob><accountId>{self.accountId}</accountId><contractId></contractId></userOperationData></mAppData>"
        data = self.aes.encrypt(data)
        
        response = requests.post(url, headers=self.headers, data=data)
        
        r = NcellResponse(response)
        
        return r
    
    def activateOffer(self, offerId):
        #: Activate the certain offer
        url = self.baseUrl + '/updateServiceRequest'
        
        data = f"<?xml version='1.0' encoding='UTF-8' standalone='yes' ?><mAppData><userOperationData><lob>{self.serviceFlag}</lob><contractId></contractId><customerId></customerId><serviceId>SUBSCRIBEAPRODUCT</serviceId><code>{offerId}</code></userOperationData></mAppData>"
        data = self.aes.encrypt(data)
        
        response = requests.post(url, headers=self.headers, data=data)
        
        r = NcellResponse(response)
        
        return r
    
    def view3gPlans(self):
        #: View available plans for 3G
        url = self.baseUrl + '/view3gPlans'
        
        data = f"<?xml version='1.0' encoding='UTF-8' standalone='yes' ?><mAppData><userOperationData><lob>{self.serviceFlag}</lob><contractId></contractId><customerId></customerId><code>{self.accountId}</code><accountId>{self.accountId}</accountId><offerId>{self.hubID}</offerId></userOperationData></mAppData>"
        data = self.aes.encrypt(data)
        
        response = requests.post(url, headers=self.headers, data=data)
        
        r = NcellResponse(response)
        
        return r


from base64 import (b64encode, b64decode)

from Crypto.Cipher import AES

class AESCipher(object):
    
    def __init__(self):
        self.key = b'zSXdd0rx59ThQlul'
        self.bs = AES.block_size

    def encrypt(self, raw):
        raw = self._pad(raw)
        #: zero based byte[16]
        iv = b'\0'*16
        cipher = AES.new(self.key, AES.MODE_CBC, iv)
        return b64encode(cipher.encrypt(raw.encode())).decode('UTF-8')

    def decrypt(self, enc):
        enc = b64decode(enc)
        #: zero based byte[16]
        iv = b'\0'*16
        cipher = AES.new(self.key, AES.MODE_CBC, iv)
        return self._unpad(cipher.decrypt(enc)).decode('utf-8')

    def _pad(self, s):
        return s + (self.bs - len(s) % self.bs) * chr(self.bs - len(s) % self.bs)

    @staticmethod
    def _unpad(s):
        return s[:-ord(s[len(s)-1:])]


import setuptools

with open("README.md", "r") as fh:
    readme = fh.read()

setuptools.setup(
    name="ncellapp", 
    version="1.0.0",
    author="Hemanta Pokharel",
    author_email="hemantapkh@yahoo.com",
    description="Unofficial Python API Wrapper of Ncell",
    long_description=readme,
    long_description_content_type="text/markdown",
    install_requires=["requests","pycryptodome"],
    url="https://github.com/hemantapkh/ncellapp",
    project_urls={
        "Documentation": "https://github.com/hemantapkh/ncellapp/blob/main/README.md",
        "Issue tracker": "https://github.com/hemantapkh/ncellapp/issues",
      },
    packages=setuptools.find_packages(),
    classifiers=[
        "Development Status :: 4 - Beta",
        "Intended Audience :: Developers",
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        'Topic :: Internet',
        'Topic :: Software Development :: Libraries',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Topic :: Utilities',
    ],
    python_requires='>=3.0',
)
